﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using nicoAE.Plugin.Interface;
using nicoAE._3DUtils;
using nicoAE.Plugin.Utils;

namespace ExtendableEditControlTest
{
    /// <summary>
    /// 頂点と頂点カラーを格納するプロパティ。
    /// </summary>
    [Serializable]
    public class VertexColorProperty : IEffectProperty
    {
        public VertexColorProperty(string name, Vertex vertex, Color color)
        {
            n = name;
            v = vertex;
            c = color;
        }

        /// <summary>
        /// プロパティの名前。
        /// </summary>
        string n = "";
        /// <summary>
        /// 頂点。
        /// </summary>
        Vertex v = new Vertex();
        /// <summary>
        /// 頂点カラー。
        /// </summary>
        Color c = new Color();

        #region IEffectProperty メンバ

        public string PropertyName
        {
            get
            {
                return n;
            }
        }

        public object Value
        {
            get
            {
                //とりあえず頂点を返してみる。
                return v;
            }
            set
            {
                v = (Vertex)value;
            }
        }

        #endregion

        public Vertex Vertex
        {
            get
            {
                return v;
            }
            set
            {
                v = value;
            }
        }

        public Color Color
        {
            get
            {
                return c;
            }
            set
            {
                c = value;
            }
        }
    }

    /// <summary>
    /// VertexColorPropertyを複数格納するプロパティ。
    /// </summary>
    [Serializable]
    public class VertexColorArrayProperty : IEffectProperty
    {
        /// <summary>
        /// length分だけVertexColorPropertyの配列を確保する。
        /// </summary>
        /// <param name="name"></param>
        /// <param name="length">VertexColorPropertyの配列の長さ。</param>
        public VertexColorArrayProperty(string name, int length)
        {
            if (length > 0)
            {
                n = name;
                properties = new VertexColorProperty[length];
            }
            else
            {
                //一応例外を飛ばしておく。
                throw new ArgumentException("Length is Zero", "length");
            }
        }

        public VertexColorArrayProperty(string name, VertexColorProperty[] vertexs)
        {
            n = name;
            properties = vertexs;
        }

        /// <summary>
        /// プロパティの名前。
        /// </summary>
        string n = "";
        /// <summary>
        /// 格納したVertexColorPropertyの配列
        /// </summary>
        VertexColorProperty[] properties = null;

        #region IEffectProperty メンバ

        public string PropertyName
        {
            get
            {
                return n;
            }
        }

        public object Value
        {
            get
            {
                return properties;
            }
            set
            {
                VertexColorProperty[] vertexs = (VertexColorProperty[])value;
                if (properties.Length == vertexs.Length)
                {
                    properties = vertexs;
                }
                else
                {
                    //一応例外を飛ばしておく。
                    throw new ArgumentException("Different Array Length");
                }
            }
        }

        #endregion

        public VertexColorProperty[] Vertexs
        {
            get
            {
                return properties;
            }
            set
            {
                if (properties.Length == value.Length)
                {
                    properties = value;
                }
                else
                {
                    //一応例外を飛ばしておく。
                    throw new ArgumentException("Different Array Length");
                }
            }
        }
        public VertexColorProperty this[int index]
        {
            get
            {
                return properties[index];
            }
            set
            {
                properties[index] = value;
            }
        }
    }

    /// <summary>
    /// 保存時にKeyFrame格納するExtendableEditControl用プロパティ。
    /// </summary>
    [Serializable]
    public class KeyFrameContainer : IEffectProperty
    {
        public KeyFrameContainer(string name)
        {
            n = name;
            dict = new Dictionary<string, KeyFrame[]>();
        }

        /// <summary>
        /// プロパティの名前。
        /// </summary>
        string n = "";
        /// <summary>
        /// KeyFrame格納用のDictionary。
        /// </summary>
        Dictionary<string, KeyFrame[]> dict = null;

        #region IEffectProperty メンバ

        public string PropertyName
        {
            get
            {
                return n;
            }
        }

        //実際に本体で使用されないので適当に実装(。
        public object Value
        {
            get
            {
                return dict;
            }
            set
            {
                dict = (Dictionary<string, KeyFrame[]>)value;
            }
        }

        #endregion

        /// <summary>
        /// KeyFrame配列を格納する。キーが存在する場合は上書きする。
        /// </summary>
        /// <param name="key">キーとなる名前。</param>
        /// <param name="keyframes">格納するKeyFrame配列。</param>
        public void AddKeyFrames(string key, KeyFrame[] keyframes)
        {
            if (dict.ContainsKey(key))
            {
                dict[key] = keyframes;
            }
            else
            {
                dict.Add(key, keyframes);
            }
        }

        /// <summary>
        /// KeyFrame配列を取得する。存在しない場合はnullを返す。
        /// </summary>
        /// <param name="key">取得するKeyFrame配列の名前。</param>
        /// <returns>取得したKeyFrame配列。</returns>
        public KeyFrame[] GetKeyFrames(string key)
        {
            if (dict.ContainsKey(key))
            {
                return dict[key];
            }
            else
            {
                return null;
            }
        }

        /// <summary>
        /// キーが格納されているかどうか調べる。
        /// </summary>
        /// <param name="key">調べるキー。</param>
        /// <returns>キーが格納されているかどうか。</returns>
        public bool ContainsKey(string key)
        {
            return dict.ContainsKey(key);
        }
    }
}
